# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oslo_log import log as logging
import oslotest.base

from designate import exceptions
from designate import objects

LOG = logging.getLogger(__name__)


class RRDataTXTTest(oslotest.base.BaseTestCase):
    def test_reject_non_quoted_spaces(self):
        recordset = objects.RecordSet(
            name='www.example.test.', type='TXT',
            records=objects.RecordList(objects=[
                objects.Record(data='foo bar'),
            ])
        )
        self.assertRaisesRegex(
            exceptions.InvalidObject,
            'Provided object does not match schema',
            recordset.validate
        )

    def test_reject_non_escaped_quotes(self):
        recordset = objects.RecordSet(
            name='www.example.test.', type='TXT',
            records=objects.RecordList(objects=[
                objects.Record(data='"foo"bar"'),
            ])
        )
        self.assertRaisesRegex(
            exceptions.InvalidObject,
            'Provided object does not match schema',
            recordset.validate
        )

    def test_multiple_strings_one_record(self):
        # these quotes do not have to be escaped as
        # per rfc7208 3.3 and rfc1035 3.3.14
        recordset = objects.RecordSet(
            name='www.example.test.', type='TXT',
            records=objects.RecordList(objects=[
                objects.Record(data='"foo" "bar"'),
            ])
        )
        self.assertRaisesRegex(
            exceptions.InvalidObject,
            'Provided object does not match schema',
            recordset.validate
        )

    def test_reject_non_matched_quotes(self):
        record = objects.TXT()
        self.assertRaisesRegex(
            ValueError,
            "TXT record is missing a double quote either at beginning "
            "or at end.",
            record.from_string,
            '"foo'
        )
        self.assertRaisesRegex(
            ValueError,
            "TXT record is missing a double quote either at beginning "
            "or at end.",
            record.from_string,
            'foo"'
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from oslo_config import cfg
from oslo_config import fixture as cfg_fixture
import oslotest.base

from designate.cmd import agent
from designate.cmd import api
from designate.cmd import central
from designate.cmd import mdns
from designate.cmd import producer
from designate.cmd import sink
from designate.cmd import worker

CONF = cfg.CONF


@mock.patch('designate.service.wait')
@mock.patch('designate.service.serve')
@mock.patch('designate.heartbeat_emitter.get_heartbeat_emitter')
@mock.patch('oslo_log.log.setup')
@mock.patch('designate.utils.read_config')
class CmdTestCase(oslotest.base.BaseTestCase):
    def setUp(self):
        super(CmdTestCase, self).setUp()
        self.useFixture(cfg_fixture.Config(CONF))

    @mock.patch('designate.agent.service.Service')
    def test_agent(self, mock_service, mock_read_config, mock_log_setup,
                   mock_heartbeat, mock_serve, mock_wait):
        CONF.set_override('workers', 1, 'service:agent')

        agent.main()

        mock_read_config.assert_called_with('designate', mock.ANY)
        mock_log_setup.assert_called_with(mock.ANY, 'designate')
        mock_service.assert_called_with()
        mock_heartbeat.assert_called()
        mock_serve.assert_called_with(mock.ANY, workers=1)
        mock_wait.assert_called_with()

    @mock.patch('designate.api.service.Service')
    def test_api(self, mock_service, mock_read_config, mock_log_setup,
                 mock_heartbeat, mock_serve, mock_wait):
        CONF.set_override('workers', 1, 'service:api')

        api.main()

        mock_read_config.assert_called_with('designate', mock.ANY)
        mock_log_setup.assert_called_with(mock.ANY, 'designate')
        mock_service.assert_called_with()
        mock_heartbeat.assert_called()
        mock_serve.assert_called_with(mock.ANY, workers=1)
        mock_wait.assert_called_with()

    @mock.patch('designate.central.service.Service')
    def test_central(self, mock_service, mock_read_config, mock_log_setup,
                     mock_heartbeat, mock_serve, mock_wait):
        CONF.set_override('workers', 1, 'service:central')

        central.main()

        mock_read_config.assert_called_with('designate', mock.ANY)
        mock_log_setup.assert_called_with(mock.ANY, 'designate')
        mock_service.assert_called_with()
        mock_heartbeat.assert_called()
        mock_serve.assert_called_with(mock.ANY, workers=1)
        mock_wait.assert_called_with()

    @mock.patch('designate.mdns.service.Service')
    def test_mdns(self, mock_service, mock_read_config, mock_log_setup,
                  mock_heartbeat, mock_serve, mock_wait):
        CONF.set_override('workers', 1, 'service:mdns')

        mdns.main()

        mock_read_config.assert_called_with('designate', mock.ANY)
        mock_log_setup.assert_called_with(mock.ANY, 'designate')
        mock_service.assert_called_with()
        mock_heartbeat.assert_called()
        mock_serve.assert_called_with(mock.ANY, workers=1)
        mock_wait.assert_called_with()

    @mock.patch('designate.producer.service.Service')
    def test_producer(self, mock_service, mock_read_config, mock_log_setup,
                      mock_heartbeat, mock_serve, mock_wait):
        CONF.set_override('workers', 1, 'service:producer')

        producer.main()

        mock_read_config.assert_called_with('designate', mock.ANY)
        mock_log_setup.assert_called_with(mock.ANY, 'designate')
        mock_service.assert_called_with()
        mock_heartbeat.assert_called()
        mock_serve.assert_called_with(mock.ANY, workers=1)
        mock_wait.assert_called_with()

    @mock.patch('designate.sink.service.Service')
    def test_sink(self, mock_service, mock_read_config, mock_log_setup,
                  mock_heartbeat, mock_serve, mock_wait):
        CONF.set_override('workers', 1, 'service:sink')

        sink.main()

        mock_read_config.assert_called_with('designate', mock.ANY)
        mock_log_setup.assert_called_with(mock.ANY, 'designate')
        mock_service.assert_called_with()
        mock_heartbeat.assert_called()
        mock_serve.assert_called_with(mock.ANY, workers=1)
        mock_wait.assert_called_with()

    @mock.patch('designate.worker.service.Service')
    def test_worker(self, mock_service, mock_read_config, mock_log_setup,
                    mock_heartbeat, mock_serve, mock_wait):
        CONF.set_override('workers', 1, 'service:worker')

        worker.main()

        mock_read_config.assert_called_with('designate', mock.ANY)
        mock_log_setup.assert_called_with(mock.ANY, 'designate')
        mock_service.assert_called_with()
        mock_heartbeat.assert_called()
        mock_serve.assert_called_with(mock.ANY, workers=1)
        mock_wait.assert_called_with()



# Copyright 2014 Hewlett-Packard Development Company, L.P.
#
# Author: Graham Hayes <graham.hayes@hpe.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from pecan import expose

from designate import exceptions


class ErrorsController(object):

    @expose(content_type='text/plain')
    @expose(content_type='text/dns')
    @expose(content_type='application/json')
    def not_found(self):
        raise exceptions.NotFound('resource not found')

    @expose(content_type='text/plain')
    @expose(content_type='text/dns')
    @expose(content_type='application/json')
    def method_not_allowed(self):
        raise exceptions.MethodNotAllowed()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from unittest import mock

import dns.message
from oslo_config import cfg

from designate import dnsmiddleware
import oslotest.base

CONF = cfg.CONF


class TestDNSMiddleware(oslotest.base.BaseTestCase):
    def setUp(self):
        super(TestDNSMiddleware, self).setUp()
        self.application = mock.Mock(name='application')
        self.dns_application = dnsmiddleware.DNSMiddleware(self.application)

    @mock.patch.object(dnsmiddleware.DNSMiddleware, 'process_request')
    def test_call(self, mock_process_request):
        request = mock.Mock()
        self.dns_application(request)

        mock_process_request.assert_called_with(request)

    @mock.patch.object(dnsmiddleware.DNSMiddleware, 'process_response')
    @mock.patch.object(dnsmiddleware.DNSMiddleware, 'process_request')
    def test_call_with_none(self, mock_process_request, mock_process_response):
        mock_process_request.return_value = None

        self.dns_application(None)

        mock_process_request.assert_called_with(None)
        mock_process_response.assert_called_with(self.application())

    def test_process_request(self):
        self.assertIsNone(self.dns_application.process_request(mock.Mock()))

    def test_process_response(self):
        response = mock.Mock()
        self.assertEqual(
            response, self.dns_application.process_response(response)
        )

    def test_build_error_response(self):
        self.assertIsInstance(
            self.dns_application._build_error_response(), dns.message.Message
        )
